@@ -30,7 +30,7 @@ const ThemeManager = new Lang.Class({
             this._changedId = 0;
         }
 
-        Main.setThemeStylesheet(null);
+        Main._cssStylesheet = null;
         Main.loadTheme();
     },
 
@@ -45,6 +45,7 @@ const ThemeManager = new Lang.Class({
                 _stylesheet = _userCssStylesheet;
             else {
                 let sysdirs = GLib.get_system_data_dirs();
+                sysdirs.unshift(GLib.get_user_data_dir());
                 for (let i = 0; i < sysdirs.length; i++) {
                     _userCssStylesheet = sysdirs[i] + '/themes/' + _themeName + '/gnome-shell/gnome-shell.css';
                     let file = Gio.file_new_for_path(_userCssStylesheet);
@@ -56,11 +57,13 @@ const ThemeManager = new Lang.Class({
             }
         }
 
-        if (_stylesheet)
+        if (_stylesheet) {
             global.log('loading user theme: ' + _stylesheet);
-        else
+            Main.setThemeStylesheet(_stylesheet);
+        } else {
             global.log('loading default theme (Adwaita)');
-        Main.setThemeStylesheet(_stylesheet);
+            Main._cssStylesheet = null;
+        }
         Main.loadTheme();
     }
 });
