@@ -53,7 +53,7 @@ function _onMenuStateChanged(menu, isOpen) {
 
     let [x, y,] = global.get_pointer();
     let actor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);
-    if (windowList.actor.contains(actor))
+    if (Me.stateObj.windowListContains(actor))
         actor.sync_hover();
 }
 
@@ -142,17 +142,19 @@ const WindowTitle = new Lang.Class({
         this._metaWindow = metaWindow;
         this.actor = new St.BoxLayout({ style_class: 'window-button-box' });
 
-        this._icon = new St.Bin({ style_class: 'window-button-icon' });
+        let app = Shell.WindowTracker.get_default().get_window_app(metaWindow);
+        this._icon = new St.Bin({ style_class: 'window-button-icon',
+                                  child: app.create_icon_texture(ICON_TEXTURE_SIZE) });
         this.actor.add(this._icon);
         this._label = new St.Label();
         this.actor.add(this._label);
 
         this._textureCache = St.TextureCache.get_default();
         this._iconThemeChangedId =
-            this._textureCache.connect('icon-theme-changed',
-                                       Lang.bind(this, this._updateIcon));
-        this._updateIcon();
-
+            this._textureCache.connect('icon-theme-changed', Lang.bind(this,
+                function() {
+                    this._icon.child = app.create_icon_texture(ICON_TEXTURE_SIZE);
+                }));
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
 
         this._notifyTitleId =
@@ -176,15 +178,6 @@ const WindowTitle = new Lang.Class({
             this._label.text = this._metaWindow.title;
     },
 
-    _updateIcon: function() {
-        let app = Shell.WindowTracker.get_default().get_window_app(this._metaWindow);
-        if (app)
-            this._icon.child = app.create_icon_texture(ICON_TEXTURE_SIZE);
-        else
-            this._icon.child = new St.Icon({ icon_name: 'icon-missing',
-                                             icon_size: ICON_TEXTURE_SIZE });
-    },
-
     _onDestroy: function() {
         this._textureCache.disconnect(this._iconThemeChangedId);
         this._metaWindow.disconnect(this._notifyTitleId);
@@ -803,7 +796,6 @@ const WindowList = new Lang.Class({
 
         Main.layoutManager.addChrome(this.actor, { affectsStruts: true,
                                                    trackFullscreen: true });
-        Main.uiGroup.set_child_above_sibling(this.actor, Main.layoutManager.trayBox);
         Main.ctrlAltTabManager.addGroup(this.actor, _("Window List"), 'start-here-symbolic');
 
         this._appSystem = Shell.AppSystem.get_default();
@@ -819,13 +811,9 @@ const WindowList = new Lang.Class({
         this._keyboardVisiblechangedId =
             Main.layoutManager.connect('keyboard-visible-changed',
                 Lang.bind(this, function(o, state) {
-                    let keyboardBox = Main.layoutManager.keyboardBox;
-                    keyboardBox.visible = state;
-                    if (state)
-                        Main.uiGroup.set_child_above_sibling(this.actor, keyboardBox);
-                    else
-                        Main.uiGroup.set_child_above_sibling(this.actor,
-                                                             Main.layoutManager.trayBox);
+                    Main.layoutManager.keyboardBox.visible = state;
+                    Main.uiGroup.set_child_above_sibling(this.actor,
+                                                         Main.layoutManager.keyboardBox);
                     this._updateKeyboardAnchor();
                 }));
 
@@ -852,6 +840,24 @@ const WindowList = new Lang.Class({
                 this._updateKeyboardAnchor();
                 this._updateMessageTrayAnchor();
             }));
+
+        this._isOnBottomMonitor = Main.layoutManager.primaryIndex == Main.layoutManager.bottomIndex;
+
+        if (this._isOnBottomMonitor) {
+            let actor = this.actor;
+            this._bottomHoverChangedId =
+                actor.connect('notify::hover', Lang.bind(Main.messageTray,
+                    function() {
+                        this._pointerInNotification = actor.hover;
+                        this._updateState();
+                    }));
+
+            this._notificationParent = Main.messageTray._notificationWidget.get_parent();
+            Main.messageTray._notificationWidget.hide();
+            Main.messageTray._notificationWidget.reparent(this.actor);
+            Main.messageTray._notificationWidget.show();
+        }
+
         this._updateMessageTrayAnchor();
 
         this._fullscreenChangedId =
@@ -987,8 +993,10 @@ const WindowList = new Lang.Class({
     },
 
     _updateMessageTrayAnchor: function() {
-        let sameMonitor = Main.layoutManager.primaryIndex == Main.layoutManager.bottomIndex;
-        let anchorY = this.actor.visible && sameMonitor ? this.actor.height : 0;
+        if (!this._isOnBottomMonitor)
+            return;
+
+        let anchorY = this.actor.visible ? this.actor.height : 0;
 
         Main.messageTray.actor.anchor_y = anchorY;
         Main.messageTray._notificationWidget.anchor_y = -anchorY;
@@ -1169,8 +1177,19 @@ const WindowList = new Lang.Class({
         global.window_manager.disconnect(this._switchWorkspaceId);
         this._switchWorkspaceId = 0;
 
-        Main.messageTray.actor.anchor_y = 0;
-        Main.messageTray._notificationWidget.anchor_y = 0;
+        if (this._bottomHoverChangedId)
+            this.actor.disconnect(this._bottomHoverChangedId);
+        this._bottomHoverChangedId = 0;
+
+        if (this._notificationParent) {
+            Main.messageTray._notificationWidget.reparent(this._notificationParent);
+            this._notificationParent = null;
+        }
+
+        if (this._isOnBottomMonitor) {
+            Main.messageTray.actor.anchor_y = 0;
+            Main.messageTray._notificationWidget.anchor_y = 0;
+        }
 
         Main.overview.disconnect(this._overviewShowingId);
         Main.overview.disconnect(this._overviewHidingId);
@@ -1188,49 +1207,41 @@ const WindowList = new Lang.Class({
     }
 });
 
-let windowList;
-let injections = {};
-let notificationParent;
+const Extension = new Lang.Class({
+    Name: 'Extension',
+
+    _init: function() {
+        this._windowList = null;
+        this._injections = {};
+    },
+
+    enable: function() {
+        this._windowList = new WindowList();
+
+        this._injections['_trayDwellTimeout'] =
+            MessageTray.MessageTray.prototype._trayDwellTimeout;
+        MessageTray.MessageTray.prototype._trayDwellTimeout = function() {
+            return false;
+        };
+    },
+
+    disable: function() {
+        if (!this._windowList)
+            return;
+
+        this._windowList.actor.hide();
+        this._windowList.actor.destroy();
+        this._windowList = null;
+
+        for (let prop in this._injections)
+            MessageTray.MessageTray.prototype[prop] = this._injections[prop];
+    },
+
+    windowListContains: function(actor) {
+        return this._windowList.actor.contains(actor);
+    }
+});
 
 function init() {
-}
-
-function enable() {
-    windowList = new WindowList();
-
-    windowList.actor.connect('notify::hover', Lang.bind(Main.messageTray,
-        function() {
-            this._pointerInNotification = windowList.actor.hover;
-            this._updateState();
-        }));
-
-    injections['_trayDwellTimeout'] = MessageTray.MessageTray.prototype._trayDwellTimeout;
-    MessageTray.MessageTray.prototype._trayDwellTimeout = function() {
-        return false;
-    };
-
-    notificationParent = Main.messageTray._notificationWidget.get_parent();
-    Main.messageTray._notificationWidget.hide();
-    Main.messageTray._notificationWidget.reparent(windowList.actor);
-    Main.messageTray._notificationWidget.show();
-}
-
-function disable() {
-    var prop;
-
-    if (!windowList)
-        return;
-
-    windowList.actor.hide();
-
-    if (notificationParent) {
-        Main.messageTray._notificationWidget.reparent(notificationParent);
-        notificationParent = null;
-    }
-
-    windowList.actor.destroy();
-    windowList = null;
-
-    for (prop in injections)
-        MessageTray.MessageTray.prototype[prop] = injections[prop];
+    return new Extension();
 }
